# This file is part of Online Services Jenkaas.
#
# Copyright 2016 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import unicode_literals
import os

from olsconfig import (
    options,
    stacks,
    stores,
)

JenkaasStore = stores.FileStore
JenkaasCmdLineStore = stores.CommandLineStore


def user_config_dir():
    return os.path.expanduser('~/.config/ols-jenkaas')


def config_file_basename():
    return 'ols-jenkaas.conf'


class JenkaasStack(stacks.Stack):

    def __init__(self):
        self.cmdline_store = JenkaasCmdLineStore()
        section_getters = [self.cmdline_store.get_sections]

        upath = os.path.join(user_config_dir(), config_file_basename())
        user_store = self.get_shared_store(JenkaasStore(upath))
        # There is only the no-name section
        section_getters.append(
            stacks.NameMatcher(user_store, None).get_sections)
        user_store
        super(JenkaasStack, self).__init__(
            section_getters, user_store, mutable_section_id=None)


def path_from_unicode(path_string):
    return os.path.expanduser(path_string)


class PathOption(options.Option):

    def __init__(self, *args, **kwargs):
        """A path option definition.

        This possibly expands the user home directory.
        """
        super(PathOption, self).__init__(
            *args, from_unicode=path_from_unicode, **kwargs)


def register(option):
    options.option_registry.register(option)


register(options.Option('launchpad.service_root', default='production',
                        help_string='''\
The launchpad service root to talk to.'''))

register(PathOption('launchpad.credentials',
                    default='~/.config/launchpad/credentials',
                    help_string='''\
The path for the launchpad OAuth token.'''))
