# This file is part of Online Services Jenkaas.
#
# Copyright 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest


from olsjenkaas import (
    config,
    launchpad,
)
from olsjenkaas.tests import (
    features,
    fixtures,
)


@features.requires(features.qastaging_token)
class TestAgainstLaunchpadQAStaging(unittest.TestCase):
    """Base class for testing against launchpad qastaging servers.

    Tests are isolated from disk and use a qastaging token to talk to
    launchpad.
    """

    def setUp(self):
        super(TestAgainstLaunchpadQAStaging, self).setUp()
        fixtures.use_qastaging(self)
        self.lp = launchpad.Launchpad(config.JenkaasStack())


@features.requires(features.launchpad_identity)
@features.requires(features.bzr_identity)
class TestQAStagingWithBzr(TestAgainstLaunchpadQAStaging):
    """Base class for testing against launchpad qastaging servers with bzr."""

    def setUp(self):
        super(TestQAStagingWithBzr, self).setUp()
        fixtures.set_bzr_identity(self, features.bzr_identity.identity)
        fixtures.set_bzr_launchpad_access(
            self, features.launchpad_identity.identity)


@features.requires(features.launchpad_identity)
@features.requires(features.git_identity)
class TestQAStagingWithGit(TestAgainstLaunchpadQAStaging):
    """Base class for testing against launchpad qastaging servers with git."""

    def setUp(self):
        super(TestQAStagingWithGit, self).setUp()
        fixtures.set_git_identity(self, features.git_identity.name,
                                  features.git_identity.email)
        fixtures.set_git_launchpad_access(
            self, features.launchpad_identity.identity)
