# This file is part of Online Services Jenkaas.
#
# Copyright 2016, 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import os

from olstests import features
from olsvms import subprocesses

# Useful shortcut
requires = features.requires


class QaStagingToken(features.Feature):
    """Credentials for qastaging launchpad."""

    def __init__(self):
        super(QaStagingToken, self).__init__()
        # FIXME: This is not available in production -- vila 2016-11-25
        self.path = os.path.expanduser('~/secrets/launchpad/qastaging')

    def _probe(self):
        # We only test that the credentials file exists here, not its validity.
        return os.path.exists(self.path)

    def feature_name(self):
        return 'An OAuth token in {} for qastaging launchpad'.format(self.path)


qastaging_token = QaStagingToken()


class BzrIdentity(features.Feature):
    """A feature capturing the current bzr whoami."""

    def __init__(self):
        super(BzrIdentity, self).__init__()
        self.identify = None

    def _probe(self):
        try:
            _, out, _ = subprocesses.run(['bzr', 'whoami'])
            self.identity = out.strip()
            return True
        except subprocesses.errors.CommandError:
            return False

    def feature_name(self):
        return 'An existing bazaar whoami'


bzr_identity = BzrIdentity()


class GitIdentity(features.Feature):
    """A feature capturing the current git user email."""

    def __init__(self):
        super(GitIdentity, self).__init__()
        self.name = None
        self.email = None

    def _probe(self):
        try:
            _, out, _ = subprocesses.run(['git', 'config', 'user.name'])
            self.name = out.strip()
            _, out, _ = subprocesses.run(['git', 'config', 'user.email'])
            self.email = out.strip()
            return True
        except subprocesses.errors.CommandError:
            return False

    def feature_name(self):
        return 'An existing git user name and email'


git_identity = GitIdentity()


class LaunchpadIdentity(features.Feature):
    """A feature capturing the current bzr launchpad login."""

    def __init__(self):
        super(LaunchpadIdentity, self).__init__()
        self.identify = None

    def _probe(self):
        try:
            _, out, _ = subprocesses.run(['bzr', 'launchpad-login'])
            self.identity = out.strip()
            return True
        except subprocesses.errors.CommandError:
            return False

    def feature_name(self):
        return 'An existing launchpad login'


launchpad_identity = LaunchpadIdentity()
