#
# Copyright 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import os


from olsjenkaas import tests
from olsjenkaas.tests import (
    features,
    fixtures,
)
from olsjenkaas.tests.branches import (
    bzr,
    git,
)


class TestBzrProposals(tests.TestQAStagingWithBzr):

    def setUp(self):
        super(TestBzrProposals, self).setUp()
        b = bzr.Branch('target')
        self.addCleanup(b.delete)
        b.create()
        b.update('Something', '''
file: foo
content
''')
        lp_name = '~{}/ols-jenkaas-test-project/target-{}'.format(
            features.launchpad_identity.identity,
            os.getpid())
        url = 'lp:' + lp_name
        b.push(url)
        self.target = self.lp.get_bzr_branch(url)
        self.assertIsNot(None, self.target)
        self.addCleanup(self.target.lp_delete)

    def test_no_mps(self):
        self.assertEqual([], list(self.target.landing_candidates))

    def test_create_mp(self):
        name = 'proposal-1-{}'.format(os.getpid())
        b = bzr.Branch(name)
        b.create_from('target')
        commit_msg = 'Something else'
        b.update(commit_msg, 'file: foo\nnewcontent\n')
        url = 'lp:~{}/ols-jenkaas-test-project/{}'.format(
            features.launchpad_identity.identity,
            name)
        b.push(url)
        lp_br = self.lp.get_bzr_branch(url)
        self.addCleanup(lp_br.lp_delete)
        proposal = lp_br.createMergeProposal(
            target_branch=self.target,
            initial_comment=commit_msg,
            commit_message=commit_msg,
            needs_review=True)
        # FIXME: We can't delete the merge proposal but deleting the target
        # branch triggers the deletion from the lp side -- vila 2017-01-03
        # self.addCleanup(proposal.lp_delete)
        self.assertEqual(commit_msg, proposal.commit_message)
        self.assertEqual('Needs review', proposal.queue_status)


class TestGitProposals(tests.TestQAStagingWithGit):

    def setUp(self):
        super(TestGitProposals, self).setUp()
        r = git.Repository('target')
        self.addCleanup(r.delete)
        r.create()
        r.update('Something', '''
file: foo
content
''')
        lp_name = '~{}/ols-jenkaas-test-project/+git/target-{}'.format(
            features.launchpad_identity.identity,
            os.getpid())
        url = 'lp:' + lp_name
        r.push(url)
        self.target = self.lp.get_git_repo(url)
        self.assertIsNot(None, self.target)
        self.addCleanup(self.target.lp_delete)

    def test_no_mps(self):
        self.assertEqual([], list(self.target.landing_candidates))

    def test_create_mp(self):
        name = 'proposal-1-{}'.format(os.getpid())
        r = git.Repository(name)
        r.create_from('target')
        commit_msg = 'Something else'
        r.update(commit_msg, 'file: foo\nnewcontent\n')
        url = 'lp:~{}/ols-jenkaas-test-project/+git/{}'.format(
            features.launchpad_identity.identity,
            name)
        r.push(url)
        lp_gr = self.lp.get_git_repo(url)
        self.addCleanup(lp_gr.lp_delete)
        master_git_ref = lp_gr.refs[0]
        self.assertEqual('refs/heads/master', master_git_ref.path)
        target_git_ref = self.target.refs[0]
        self.assertEqual('refs/heads/master', target_git_ref.path)
        proposal = master_git_ref.createMergeProposal(
            merge_target=target_git_ref,
            initial_comment=commit_msg,
            commit_message=commit_msg,
            needs_review=True)
        # FIXME: We can't delete the merge proposal but deleting the target
        # branch triggers the deletion from the lp side -- vila 2017-01-03
        # self.addCleanup(proposal.lp_delete)
        self.assertEqual(commit_msg, proposal.commit_message)
        self.assertEqual('Needs review', proposal.queue_status)
